# -*- coding: utf8 -*-

"""Helper components for web links."""

from __future__ import annotations

__all__ = ['open_link', 'NavigatorLink']

import collections.abc
import typing
import urllib.parse

from qtpy import QtCore
from qtpy import QtGui

from anaconda_navigator.utils import telemetry

if typing.TYPE_CHECKING:
    import typing_extensions


def open_link(url: str | QtCore.QUrl, *, origin: str | None = None) -> None:
    """Open a :code:`url` in an external web browser."""
    qt_url: QtCore.QUrl
    if isinstance(url, QtCore.QUrl):
        url, qt_url = url.toString(), url
    else:
        qt_url = QtCore.QUrl(url)

    if url.startswith('navigator://'):
        raise ValueError('Navigator links must be processed separately')

    QtGui.QDesktopServices.openUrl(qt_url)

    details: dict[str, typing.Any] = {'url': url}
    if origin is not None:
        details['origin'] = origin
    telemetry.ANALYTICS.instance.event('redirect', details)


class NavigatorLink(typing.NamedTuple):
    """Details on a :code:`navigator://` link."""

    path: collections.abc.Sequence[str]
    query: collections.abc.Mapping[str, str]

    @classmethod
    def parse(cls, url: str) -> typing_extensions.Self:
        """Parse a :code:`navigator://` link."""
        parts: list[str] = url.split('://', 1)
        if len(parts) > 1:
            if parts[0] != 'navigator':
                raise ValueError('url is not a `navigator://` link')
            parts = parts[1:]

        parts = parts[0].split('?', 1)

        query: dict[str, str] = {}
        if len(parts) >= 2:
            query.update(urllib.parse.parse_qsl(parts[1], keep_blank_values=True))

        return cls(
            path=tuple(map(urllib.parse.unquote, parts[0].split('/'))),
            query=query,
        )



# -*- coding: utf-8 -*-

"""Structure to store API endpoint URLs."""

__all__ = ['EndpointCollection']

import typing
from urllib import parse


class EndpointCollection(typing.Mapping[str, typing.Any]):
    """
    Collection of API endpoint URLs.

    :param base_url: Root URL, parent for all API endpoints.
    :param endpoints: Collection of links, that are relative to the `base_url`.

                      Hierarchies are also supported - collection of links might be used besides relative urls.
    """

    __slots__ = ('__base_url', '__endpoints')

    def __init__(self, base_url: str, endpoints: typing.Mapping[str, typing.Any]) -> None:
        """Initialize new :class:`~ApiCollection` instance."""
        self.__base_url: typing.Final[str] = base_url

        key: str
        value: typing.Any
        data: typing.Dict[str, typing.Union[str, EndpointCollection]] = {}
        for key, value in endpoints.items():
            if isinstance(value, str):
                data[key] = parse.urljoin(self.__base_url, value)
            elif isinstance(value, typing.Mapping):
                data[key] = EndpointCollection(base_url=base_url, endpoints=value)
            else:
                raise TypeError(f'endpoint must be str or Mapping, not {type(value).__name__}')

        self.__endpoints: typing.Final[typing.Mapping[str, typing.Union[str, EndpointCollection]]] = data

    @property
    def base_url(self) -> str:  # noqa: D401
        """Root URL, parent for all API endpoints."""
        return self.__base_url

    def __getattr__(self, key: str) -> typing.Any:
        """Retrieve endpoint URL as an attribute."""
        try:
            return self.__endpoints[key]
        except KeyError:
            raise AttributeError(f'{type(self).__name__!r} object has no attribute {key!r}') from None

    def __getitem__(self, key: str) -> typing.Any:
        """Retrieve endpoint URL as an item."""
        return self.__endpoints[key]

    def __len__(self) -> int:
        """Retrieve total number of registered endpoints."""
        return len(self.__endpoints)

    def __iter__(self) -> typing.Iterator[str]:
        """Iterate through endpoint names."""
        return iter(self.__endpoints)
